/* feb  2  1999: The GoSWISH front end to the SWISH search engine

  GOSWISH can be called in a number of different manners:

        a) As a cgi-bin script, or sre-http addon, to create a
           swish index
        b) Called as a Detached program, in order to create a
           description cache
        c) As a cgi-bin script (or sre-http addon) to search a swish index 
           (and possibly a description   cache)
        d) As a cgi-bin script (or sre-http addon) to display list of current
          "search forms" (stored in goswish.ind)
    
 Regardless of how you use it .... you'll need to set a few parameters below
 (the same settings are use in all modes). 

*/

signal on error name foo10 ; signal on syntax name foo10

/** BEGIN USER CONFIGURABLE PARAMETERS */
/* If you do not set these parameters (i.e.; set them =""); then
   defaults will be used.  These defaults will make some sense if
   you run this as an sre-http addon, but will probably be 
   barely tolerable if you run this as a cgi-bin script.
*/

/* INDEX_DIR should be the fully qualified name of the directory
   used for storing SWISH Index Files. It should also contain SWISH.EXE
   (or, SWISH should be in your DPATH).
   If called as an SRE-http addon, the default is the SRE-http DATA directory.
   If called as CGI-BIN, the default is the current directory (which is probably
   inappropriate).
*/

index_dir='\GOSERVE\SWISH'


/* WWW_DIR is the root of your web directories. 
   If called as an SRE-http addon, the default is the GoServe data directory.
   If called as CGI-BIN, the default is the current directory (which is probably
   inappropriate). 
   You can override this parameter with the  WWWDIR option.
*/

www_dir='\www' 

/* SRE-http option:
     NEED_PRIVS is a set of "privileges" that permit creation of a swish
     index.  That is, to create a swish index, the client must have ONE 
     of these privileges (not all of, but at least one of). 
     If she doesn't have one of these
     privileges, an authorization request will be returned.
     If this is called as a cgi-bin script, NEED_PRIVS is ignored.
     If need_privs contains an *, then need_privs is ignored (open access)
*/
need_privs='*'
   

/** END USER CONFIGURABLE PARAMETERS */

goswish:
parse arg  ddir, tempfile, reqstrg,list,verb ,uri,user, ,
          basedir ,workdir,privset,enmadd,transaction,verbose, ,
         servername,host_nickname,homedir,xx,semqueue,prog_file


if abbrev(strip(ddir),'*DESCRIBE')=1 then do 
    verb='DESCRIBE'
    parse var ddir . daswifile ','darealdir','darepwith','datype
    list0=list
    darepwith=replacestrg(darepwith,'\','/','ALL')
end /* do */

if www_dir=0 then www_dir=""                    /* double check directories */
if index_dir=0 then index_dir=""
privset=translate(privset)

/* check for CGI-BIN call */
is_cgi=0
if verb="" then do    /* is it cgi-bin? */
   method = value("REQUEST_METHOD",,'os2environment')
   if method="" then do
     say "This script addon is not meant to be run in standalone mode "
     exit
   end  /* Do */
   else do
      is_cgi=1
      if method='GET' then do
          list=value("QUERY_STRING",,'os2environment')
      end
      else do
         tlen = value("CONTENT_LENGTH",,'os2environment')
         list=charin(,,tlen)
      end /* do */
      servername=value("SERVER_NAME",,'os2environment')
   end
   if www_dir='' then www_dir=directory()
   if index_dir='' then index_dir=directory()
   prog_file=''
end


index_dir=strip(index_dir,'t','\')
okk=dir_exists(index_dir)
if okk=0 then do
    foo=is_error(' Sorry, bad index directory: 'index_dir)
    exit
end /* do */
www_dir=strip(www_dir,'t','\')
okk=dir_exists(www_dir)
if okk=0 then do
    foo=is_error(' Sorry, bad www directory: 'www_dir)
    exit
end /* do */

/* set defaults */

deflist='sel swifile repwith  indexname  indexadmin indexdescription indexpointer ',
        'extlist extlist_nofollow ignorelimit ignorewords mode ',
        ' makesummary summaryfile describefile htmls ',
        'fr_directory fr_title  fr_filename fr_pathname wwwdir watch searchdoc '
deflist=translate(deflist)

vs.!verbose=3
vs.!wwwdir=www_dir
vs.!sel='/'
vs.!searchdoc=''
vs.!repwith=0
vs.!watch=0
vs.!indexname=0 
vs.!indexdescription=0
vs.!indexadmin='Created by 'user
vs.!indexpointer=servername
vs.!makesummary=0

vs.!extlist='.htm .txt .gif .xbm .jpg .doc .sht .html .shtml '
vs.!extlist_nofollow=' .gif .xbm .jpg '
vs.!ignorelimit='50 100'
vs.!ignorewords='SwishDefault'

vs.!fr_pathname='contains admin testing demo trash construction PRIVATE private confidential '
vs.!fr_filename='contains # % ~ .bak .orig .old old. '
vs.!fr_title='contains construction example pointers '
vs.!fr_directory='contains .htaccess'

vs.!htmls=" HTM HTML SHTML SHT "
vs.!describefile='DESCRIBE.TXT'

/* parse options */
list0=list
do until list=""
   parse var list aw '&' list
   parse var aw a1 '=' a2
   a1=translate(strip(a1))
   if wordpos(a1,deflist)=0 then iterate  /* not a valid option */
   aa='!'||a1
   a2=strip(decodekeyval(translate(a2,' ','+')))
   if a2="" | a2=0 then iterate                /* blank,  don't change */
   vs.aa=a2
end /* do */
/* fix up parameters */

if abbrev(translate(vs.!mode),'S')=1 then do   /* relatively simple cgi-bin search of swish */
  foo=search_it(list0)
  return 0
end /* do */

if abbrev(translate(Vs.!mode),'L')=1  then do  /* show list of "search forms */
   foo=show_forms(list0)
   return 0
end /* do */

/* if search mode, call srch_index, then leave */
if abbrev(translate(vs.!mode),'S') then do
   call srch_index
   return 0     
end /* do */

/* else== making an index */


if is_cgi=0 & verb<>'DESCRIBE' then do       /* sre-http addon mode -- making index requires privileges */
  isp=0
  if wordpos('*',need_privs)>0 then do
     isp=1
  end /* do */
  else do
    do mm=1 to words(need_privs)
        aww=strip(translate(word(need_privs,mm)))
        if wordpos(aww,privset)>0  then do
            isp=mm ; leave
        end /* do */
    end /* do */
  end
  if isp=0 then do
      'header add WWW-Authenticate: Basic Realm=<!SWISH_INDEX>'  /* challenge */
       return sref_response('unauth', "You do not have SWISH Index creation rights ",tempfile,servername)
  end  /* Do */
  if  method="GET" then    parse var uri . '?' list   
  if www_dir='' then www_dir=ddir
  if index_dir='' then index_dir=get_value('workdata_dir')
end
if verb<>"DESCRIBE" then do
  vs.!swifile=mk_filename(index_dir'\index','.swi')
end

vs.!wwwdir=strip(translate(vs.!wwwdir,'\','/'),'t','\')
if dir_exists(vs.!wwwdir)=0 then do
    foo=is_error(' Sorry, no such target directory for search document: 'vs.!wwwdir)
    exit
end

vv=translate(vs.!makesummary)
select
   when wordpos(vv,'N NO 0')>0 then
    vs.!makesummary=0
   when wordpos(vv,'Y H HTML 1')>0 then
     vs.!makesummary=1
   when wordpos(vv,'A ALL 2')>0 then
     vs.!makesummary=2
  otherwise
     vs.!makesummary=0
end

tt=""
do until vs.!htmls=""
   parse var vs.!htmls a1 vs.!htmls ;a1=strip(a1)
   tt=tt||' '||strip(a1,'l','.')
end /* do */
vs.!htmls=strip(tt)

tt=""
do until vs.!extlist=""
   parse var vs.!extlist a1 vs.!extlist ;a1=strip(a1)
   tt=tt||' .'||strip(a1,'l','.')
end /* do */
vs.!extlist=strip(tt)

tt=""
do until vs.!extlist_nofollow=""
   parse var vs.!extlist_nofollow a1 vs.!extlist_nofollow ;a1=strip(a1)
   tt=tt||' .'||strip(a1,'l','.')
end /* do */
vs.!extlist_nofollow=strip(tt)


if wordpos(translate(vs.!watch),'1 YES Y')>0 then vs.!watch=1
if is_cgi=1 then vs.!watch=0    /* cgi-bin monitoring of output is too hard */

if pos(':',vs.!sel)>0 then do
     foo=is_error(' Sorry, do not enter a fully qualified directory ( 'vs.!sel)
     exit
end /* do */
vs.!selorig=vs.!sel

vsel=vs.!sel
vs.!sel='' 
do forever              /* fix up, converting / to \ */
   if vsel='' then leave
   parse var vsel a1 vsel ; a1=strip(a1)
   vs.!sel=vs.!sel'  \'||strip(translate(a1,'\','/'),,'\')
end /* do */

if datatype(vs.!verbose)<>'NUM' then vs.!verbose=3
if vs.!verbose>3 | vs.!verbose<0 then vs.!verbose=3

if pos(':',vs.!swifile)=0 & abbrev(vs.!swifile,'\')=0 then
  vs.!swifile=index_dir'\'vs.!swifile

if verb<>"DESCRIBE" then do
   parse var vs.!swifile aa '.' .   /* the description cache file (if needed) */
   vs.!summaryfile=aa||'.dct'
end

/*  SPECIAL make Summary CALL? */
if verb="DESCRIBE" then   do    /* make a summary mode ? */
   swf=daswifile ; vs.!swifile=swf
   parse var vs.!swifile aa '.' .   /* the description cache file (if needed) */
   vs.!summaryfile=aa||'.dct'
   vs.!makesummary=datype
   vs.!watch=0 ; vs.!mode='SUMMARY'
   vs.!repwith=darepwith
   vs.!realdir=darealdir

   do forever           /* wait until vs.!swifile is avaiable */   
       if stream(swf,'c','query exists')<>"" then leave
       call syssleep 2
   end /* do */
   call syssleep 2
   fpp=make_the_summary()
   exit
end

/* nope, full swish index (or search mode ) */
/* the search-form document to create (and a link to it */
usedoc=translate(strip(vs.!searchdoc),' ','+')
select
   when usedoc="" then do                /* none specified */
     arf0="SEARCH"
     aform=mk_filename(vs.!wwwdir'\'arf0,'.HTM')  /* the search-form document */
     doob=translate(substr(aform,length(vs.!wwwdir)+1),'/','\')
  end
  when pos(':',usedoc)=0 & abbrev(usedoc,'/')=0 then do  /* a relative filename*/
     arf0=strip(translate(USEDOC,'\','/'))
     if right(arf0,1)='\' then arf0=arf0||'SEARCH'
     aform=mk_filename(vs.!wwwdir'\'arf0,'.HTM')  /* the search-form document */
     doob=translate(substr(aform,length(vs.!wwwdir)+1),'/','\')
  end
  otherwise do
     parse var  usedoc aform doob
  end
end
yeek=filespec('D',aform)||filespec('P',aform)
if dir_exists(yeek)=0 then do
     foo=is_error(' Sorry, bad search-document name: 'aform)
     exit
end /* do */
fff=sysfiledelete(aforM)

if vs.!indexdescription=0 | vs.!indexdescription=' ' then 
   vs.!indexdescription=vs.!selorig' on ' servername
if vs.!indexname=0 | vs.!indexname=' ' then 
  vs.!indexname='Index of 'vs.!selorig

/* write the response to client */

crlf='0d0a'x
dafile='<html><head><title>Swish Configuration File</title></head></body>'crlf
dafile=dafile||'<a name="TOP"><h2>Creating a Swish Index </h2></a>'crlf
dafile=dafile||' Creating a SWISH index of 'vs.!selorig
dafile=dafile||'<h3>The search form</h3> 'crlf
if doob<>'' then
  dafile=dafile||' The search-form document is: <a href="'doob'">'aform||'</a>'crlf
else
  dafile=dafile||' The search-form document is: 'aform||crlf

xx=write_form(aform,vs.!swifile)  /* write stuff to search-form document */

dafile=dafile||'<br><em>Use it to search the 'vs.!swifile' SWISH index'
if vs.!makesummary>0 then 
  dafile=dafile||', or the 'vs.!summaryfile ' descriptions file</em>'crlf
else
  dafile=dafile||'.</em>'crlf

if vs.!watch=0  then dafile=dafile||'<p> <b>You may need to wait for a few minutes </b> (until SWISH completes 'vs.!swifile')'crlf

cnffile=mk_filename(index_dir'\swish','.con')

dafile=dafile||'<hr><h3> The configuration file </h3> 'crlf
dafile=dafile||' The following configuration file ('cnffile ') is  used to generate the <b>' vs.!swifile '</b> <em> SWISH index</em><br>'

if is_cgi=0 then do
  fii=value('SREF_PREFIX',,'os2environment')
  if fii='' then do             /* 1.2 */
     if fex>0 then  foo=sref_expire_response(fex,0,,'Y')
     'SEND TYPE  text/html '
     'var name dafile'
  end
  else do
       fii=sref_multi_send(dafile,'text/html','1S')  /* sre ver 1.3 */
  end /* do */
end
else do
   say 'Content-Type: text/html'
   say ""
   say dafile           /* cgi-bin output */
end

dafile=write_conf()      /* create the swish configuration file, and run swish */

if vs.!watch=0 then do
  dafile='<br><em> Construction of 'vs.!swifile' may take a few minutes.</em><br>'
end /* do */

/* make a summary (here, with echoing; or via a detach */
if vs.!makesummary>0 then do
   swf=vs.!swifile
   if vs.!watch=1 then do
     call write_her('<hr><pre>')
     aa=time('r')
     do forever           /* wait until vs.!swifile is avaiable */   
       if stream(swf,'c','query exists')="" then do
          if is_cgi=0 then do
             if vs.!watch=1 then call write_her('Waiting for 'swf' '||time('e'))
              if result<0  then return 0
          end /* do */
          call syssleep 1
          iterate
       end /* do */
       else do
         if vs.!watch=1 then call write_her('</pre><h3>Creating Descriptions </h3>')
         leave
       end
     end /* do */
     fpp=make_the_summary()
   end                  /* watch mode */
   else do              /* non-watch -- use detach */
      if prog_file<>" " then
          xxx=prog_file
      else
          xxx=index_dir'\goswish'
      tmp1=replacestrg(vs.!repwith,'/','\','ALL')
      arglist='*DESCRIBE 'vs.!swifile','vs.!realdir','tmp1','vs.!makesummary
      if is_cgi=0 then do
         address cmd 'detach  ' xxx ' ' arglist
      end
      else do
         bubba='"Descriptions of 'vs.!selorig'" /C /MIN  'xxx ' 'arglist ' > nul '
         address cmd 'start 'bubba
      end /* do */

   end /* do */
end

call write_her('<hr><a href="#top">Top of document</a>')

dafile='</pre></body></html>'
call write_her(dafile)

if is_cgi=0 then  do
  if fii='' then   /* sre 1.2 */
   'SEND COMPLETE '
  else
     foo=sref_multi_send(' ',,'1E')  /* sre 1.3 */
end

/* write info to the "index "*/
tbo=date('n')' 'time('n')
if vs.!makesummary>0 then tbo=tbo' (w/summaries) '
aline=doob','aform','vs.!swifile','tbo','vs.!indexname
goindx=index_dir'\goswish.ind'
if stream(goindx,'c','query exists')<>"" then do
    oo=translate(stream(goindx,'c','open'))
    if abbrev(oo,'READY')=0 then return 0        /* can't open, give up */
    aa=charin(goindx,1,chars(goindx))
    aa=strip(aa,'t','1a'x)
end
else do
    aa=''
end /* do */
aa=aline||'0d0a'x||aa
aa2=charout(goindx,aa,1)
foo=stream(goindx,'c','close')
return 0

/***********************/
/* write the configuration file, with parameters */
write_conf:procedure expose vs. index_dir www_dir servername is_cgi ,
                enmadd ddir transaction homedir host_nickname cnffile

if is_cgi=0 & vs.!wwwdir='' then do             /* use a virtual dir */
    vsel=vs.!sel ; realdir='' ; realsel=''
    do forever
       if vsel='' then leave
       parse var vsel eek1 vsel ; eek1=strip(eek1)
       if pos('.',eek1)=0 then eek1=eek1||'/' 
       eek1=translate(eek1,'/','\')
       realdir9=sref_do_virtual(ddir,eek1,enmadd,0,trans,homedir,host_nickname)
       realdir=realdir' 'strip(realdir9,'t','\')
       realsel=realsel' 'eek1
   end
end
else do                                                                            
    vsel=vs.!sel ; realdir=''; realsel=''
    do forever
       if vsel='' then leave
       parse var vsel eek1 vsel ; eek1=strip(eek1)
       realdir=realdir' 'strip(strip(vs.!wwwdir,'t','\')||eek1,'t','\')
       eek1=translate(eek1,'/','\')
       realsel=realsel' 'eek1
   end
end

crlf='0d0a'x

do ii=1 to 40
   conf.ii=' '
end

conf.0=38
conf.1='#Auto-generated SWISH configuration file '||date('n')' 'time('n')
conf.3=' '
conf.4='# space delimited list of files/directories to index '
conf.5='IndexDir 'realdir
conf.6=' '
conf.7='# the generated index file '
conf.8='IndexFile 'vs.!swifile
conf.9=' '
conf.10='#some identification info '

conf.11='IndexName 'vs.!indexname
conf.12='index description 'vs.!indexdescription

conf.13='indexpointer ' vs.!indexpointer
conf.14='indexadmin ' VS.!indexadmin
conf.15=' '
conf.16='# Only files with these suffixes will be indexed.'
conf.17='IndexOnly  ' vs.!extlist
conf.18='# but do not look at contents of these files '
conf.19='NoContents 'vs.!extlist_nofollow 

if vs.!repwith=0 | vs.!repwith=' ' then do
  repwith=''
  do mm=1 to words(realsel)
     repwith=repwith||' http://'||servername||strip(word(realsel,mm))
  end
end /* do */
else do
   repwith=vs.!repwith
end

vs.!repwith=repwith
vs.!realdir=realdir

conf.21='# this convert fully qualified file references into urls '
conf.22=''
do mm=1 to words(realdir)
   realdir1=strip(word(realdir,mm)); repwith1=strip(word(repwith,mm))
   conf.22=conf.22|| ,
        'ReplaceRules replace "'realdir1'"  "'||repwith1||'" '
   if mm<>words(realdir) then conf.22=conf.22||'0d0a'x
end /* do */
/* conf.22='ReplaceRules replace "'realdir'"  "'||repwith||'" ' */

conf.25='# these are used to not search files and directories '
if vs.!fr_pathname<>0 & vs.!fr_pathname<>' ' then conf.26='FileRules pathname   ' vs.!fr_pathname
if vs.!fr_filename<>0 & vs.!fr_filename<>" " then conf.27='FileRules filename   'vs.!fr_filename
if vs.!fr_title<>0 & vs.!fr_title<>' 'then conf.28='FileRules title  'vs.!fr_title
if vs.!fr_directory<>0 & vs.!fr_directory<>' 'then conf.29='FileRules directory  ' vs.!fr_directory
conf.31='# ignore certain common words '
conf.32='IgnoreLimit ' vs.!ignorelimit
conf.33='IgnoreWords ' vs.!ignorewords

conf.35='# verbosity, and an os/2 necessary switch '
conf.36=' IndexReport ' vs.!verbose
conf.37=' followSymLinks no '


aboo=""
do mm=1 to conf.0
 call lineout cnffile,conf.mm
  aboo=aboo||conf.mm||'0d0a'x
end /* do */
call lineout cnffile

aboo='<pre>'aboo'</pre>'
call write_her(aboo)

/* cgi bin? */
if is_cgi=1 then do
   xxx=index_dir'\swish'
   if stream(xxx'.exe','c','query exists')="" then do
      say "<b> ERROR: Could not find " xxx'.exe </b>'
      return 0
   end
   address cmd 'START "Swish Index for 'VS.!selorig '" /C /MIN 'xxx ' -c  'cnffile ' > nul'  
/*  address cmd 'detach 'xxx ' -c  'cnffile ' > nul' */
   return 1
end

/* sre-http.. */
/* monitor output via RXQUEUE */
useq=''
if vs.!watch<>0 then do
  oldq=rxqueue('g')
  useq=rxqueue('c','SWISHQ')
  foo=rxqueue('s',useq)
  do queued(); pull .; end                   /* flush */
end

cnfdir=strip(filespec('d',cnffile)||filespec('p',cnffile),'t','\')
fdr=directory(cnfdir)
address cmd 'detach swish -c 'cnffile ' | rxqueue  "'useq'"'
aa=directory(fdr)
if vs.!watch=0 then return 0
/* monitor output of swish */

aboo='<hr><h3> SWISH Results </h3><pre>'||crlf
call write_her(aboo)
if result<0 then do
      'string 'crlf||'</pre> .. broken connection, but SWISH is still running)<p> '
      return 0
end

oo=time('r')
ipo=0
do forever
  ipo2=0 ;bbb=""
  if queued()>0 then do
     parse pull aline
     if TRANSLATE(strip(aline))='INDEXING DONE!' then leave
     ipo2=ipo2+1 ;ipo=0
     bbb=bbb||aline||crlf
     if ipo2>10 then do
         call write_her(bbb)
         bbb=""
        foo=time('r')
     end
  end
  if length(bbb)>0 then do
        call write_her(bbb)
        foo=time('r')
  end
  if time('e')>15 then do
      'string 'crlf||'</pre> ... (monitoring now ending, but SWISH is still running)<p> '
      return 0
  end /* do */
end

foo=rxqueue('d',useq)
foo=rxqueue('s',oldq)

call write_her('</pre>')

return 1


/**************************/
/* create unused file name, numbered from 1....*/
mk_filename:procedure
parse arg aname,aext
aname=strip(aname); aext=strip(aext)
PARSE var aname aname '.' aext0
if aext0<>'' then aext='.'aext0
do mm=1 to 999
    f1=aname||mm||aext
    if stream(f1,'c','query exists')="" then do
       return f1
    end /* do */
end /* do */
return f1       /* use name9999.ext in a pinch */


/************************************************/
/* procedure from TEST-CGI.CMD by  Frankie Fan <kfan@netcom.com>  7/11/94 */
DecodeKeyVal: procedure
  parse arg Code
  Text=''
  Code=translate(Code, ' ', '+')
  rest='%'
  do while (rest\='')
     Parse var Code T '%' rest
     Text=Text || T
     if (rest\='' ) then
      do
        ch = left( rest,2)
        if verify(ch,'01234567890ABCDEF')=0 then
           c=X2C(ch)
        else
           c=ch
        Text=Text || c
        Code=substr( rest, 3)
      end
  end
  return Text


/*********/
/* return error message */
is_error:procedure expose is_cgi
parse arg amess
if  is_cgi=0 then do
     'string ' amess
end
else do
   say 'Content-type: text/plain'
   say
   say amess
end
return 1


/***********/
/* generate form that calls the "search in a swish index" document */
write_form:procedure expose vs. is_cgi
parse arg tofile,swifile

do mmm=1 to 50
   frm.mmm=''
end /* do */
frm.1='<!doctype html public "-//IETF//DTD HTML 2.0//EN">'
frm.2='<html><head><title>Search an index  </title></head><body>'
frm.3='<a name="top"><h1>Search an index of 'vs.!selorig'</h1></a>'
frm.4='Please enter a list of words to search for (note the options in the'
frm.5='lower portion of this page).<p>'

if is_cgi=1 then
   frm.6='<FORM ACTION="/cgi-bin/goswish" METHOD="GET">'
else 
  frm.6='<FORM ACTION="/srchindx" METHOD="POST">'

frm.7='<INPUT TYPE="hidden" NAME="index"  VALUE="'swifile'">'
frm.8='<INPUT TYPE="hidden" NAME="text_descrip_file"  VALUE="'vs.!summaryfile'">'
frm.9=' '
if is_cgi=0 then frm.10='<INPUT TYPE="hidden" NAME="comment"  ' ,
    ' VALUE="Search is against all complete words in your documents."> '

frm.11='Enter search string: <INPUT TYPE="text" NAME="keyword"  VALUE="help"  SIZE=35 MAXLENGTH=75>'
frm.12='<br>'
frm.13='<br>Search is case insensitive. You can use * to match the <beginning of words <em>(for example, SHIP* will match <tt>SHIP</tt> and <tt>ShipMate</tt>)</em>'
frm.14='<br><INPUT TYPE="submit" VALUE="Search now"> <INPUT TYPE="reset" VALUE="reset values">'
frm.15='<h3> Options: </h3>'
frm.16='<INPUT TYPE=RADIO NAME="cond" VALUE="OR">Match any ||'
frm.17='<INPUT TYPE=RADIO NAME="cond" checked VALUE="AND">Match all ||'
frm.18=''
frm.19='<INPUT TYPE=RADIO NAME="cond" VALUE="NOT">Match first, but not others<br>'
frm.20='<INPUT TYPE=RADIO NAME="option1" VALUE="-m 10">Best 10 matches ||'
frm.21='<INPUT TYPE=RADIO NAME="option1" checked VALUE="-m 20">Best 20 matches ||'
frm.22='<INPUT TYPE=RADIO NAME="option1" VALUE="-m 40">Best 40 matches <br>'
frm.23='<INPUT TYPE=RADIO NAME="option1" VALUE="-m 150">Best 150 matches <br>'

i6=6
if vs.!makesummary=0 then i6=4
frm.24='<SELECT NAME="option2" SIZE='i6'> '
frm.25='<OPTION SELECTED value="" >Search all documents'
frm.26='<OPTION value="-t+HB">Search HTML documents: text'
frm.27='<OPTION value="-t+ehtc">Search HTML documents: "descriptive" elements'
frm.28='<OPTION value="-t+t">Search HTML documents: &lt;TITLE&gt; only'
if vs.!makesummary>0 then do
  frm.29='<option value="summary"> Search "summaries" only'
  frm.30='<OPTION value="file">Search File names only'
  frm.33='<input type="checkbox" name="summary" value="1">Display a summary '
end
frm.31='</SELECT><p>'
/* Too slow, so don't make it automatically available.
 if is_cgi=0 then frm.34='<br> <input type="checkbox" name="exists" value=1">   Check that document exists <br>'
*/

frm.35='<input type="hidden" name="HEADER" value="Search of 'vs.!selorig'">'
frm.36='<input type="hidden" name="MODE" value="S"> '
if is_cgi<>1 then frm.36=frm.36||' <input type="Hidden" name="START" value="1+0">'

frm.38='</FORM>'
frm.37='<p><INPUT TYPE="submit" VALUE="Search now">'

frm.39='<hr><a href="#top">Top of document</a>'
frm.40='</ul><hr><h4>Description</h4>'vs.!indexdescription'</body></html>'

do mm=1 to 40
   call lineout tofile,frm.mm
end
call lineout tofile

return 1

foo10:
say " GOSWISH: error at " sigl rc 
exit 0


/* See if directory exists , 0=no 1=yes*/
dir_exists:procedure
parse upper arg lookfor
lookfor=strip(lookfor,'t','\')

adrive=filespec('d',lookfor)       /* does drive exist? */
if adrive<>"" then do
  oo2=sysdrivemap(,'used')
  if pos(translate(adrive),translate(oo2))=0 then return 0   /* no such drive */
end
eek=lastpos('\',lookfor)
if eek>0 then do
  lookfor1=substr(lookfor,eek+1)
  foo=delstr(lookfor,eek)
end
else do
   return 1       /* it's a root dir */
end /* do */
foo=foo'\*.*'
aa=sysfiletree(foo,'eek','DO')
do mm=1 to eek.0
   if translate(filespec('n',eek.mm))=lookfor1 then do 
       return 1
   end /* do */
end /* do */
return 0

/* ----------- */
/* get environment value, possibly host specific */
/* ------------ */
get_value: procedure expose enmadd host_nickname
parse arg vname,hname0
if hname0=0 then
        hname=' '
else
    hname=strip(host_nickname)

vname=strip(vname) ;
if hname<>' ' then do
   aval=value(enmadd||vname||'.'||hname,,'os2environment')
   if aval<>' ' Then
        return aval
end
aval=value(enmadd||vname,,'os2environment')
return aval



/********/
/* say or send , or not, adesc */
write_her:procedure expose vs. is_cgi fii
parse arg axx

if upper(fii)='FII' then fii=value('SREF_PREFIX',,'os2environment')

if is_cgi=1 then do
  say axx
end
else do
  if fii='' then do
    'VAR 'Axx
  end
  else do
    foo=sref_multi_send(axx)
    if multi<0 then return -1
  end
end
return 1

/*************************************/
/*************************************/
/*************************************/
/* create a "description cache"  */
make_the_summary:procedure expose vs. list0


/* parameters */
htmls=vs.!htmls
nocontents=vs.!extlist_Nofollow
outname=vs.!summaryfile
fdescribe=vs.!describefile
daindx=vs.!swifile
defdir=vs.!wwwdir
crlf='0d0a'x


/* These are used as default "summaries" */
gmess.1=' File not available'
gmess.2=' Summary not available'
gmess.3=' Explicit summary not available'
gmess.4=' No summary available'

/* This is the character used to signal "continuation of a description"
I.e. (assuming continuation_flag='|'
FOOBAR.TXT   This is the description of foobar.txt
 |           And this is the second line.
Note that the | should be the first non space character */
continuation_flag='|'

if vs.!repwith=0 | vs.!repwith='' then do  
  nreps=0
end /* do */
else do
   nreps=words(vs.!repwith)
   do nbb=1 to nreps   
      reprules.nbb.!new=strip(translate(word(vs.!repwith,nbb)))
      reprules.nbb.!orig=strip(translate(word(vs.!realdir,nbb)))
   end
end /* do */

call get_confile        /* read filenames from .swi file */
call get_filelist_info

if vs.!watch=1 then call write_her(" Creating descriptions for " filelist.0 " files <ol> ")
latesd.!dir=' '
do m=1 to filelist.0
   bigd.m=translate(filelist.m.!original,'/','\')
   bigd.m.!title=filelist.m.!title
   bigd.m.!size=filelist.m.!size
   bigd.m.!YES=strip(make_summary(filelist.m,filelist.m.!type,vs.!makesummary))
   bigd.m.!gotit=yaman  /*0-none, 1=created, 2=explicit */
   if vs.!watch=1 then do
     if yaman>0 then 
         call write_her(' <li> 'bigd.m', 'bigd.m.!title)
     else
         call write_her(' <li> 'bigd.m', No Description is Available')
   end
end /* do */

bigd.0=filelist.0

div=' &^%^& '
div2=' #$*~~#$* '
allf=""
do ii=1 to bigd.0
   aa=bigd.ii.!gotit||div||bigd.ii||div||bigd.ii.!title||div||bigd.ii.!size|| , 
        div||bigd.ii.!yes
   allf=allf||aa
   if ii<>bigd.0 then allf=allf||div2
end /* do */
sike=charout(outname,allf,1)
sike=stream(outname,'c','close')

if vs.!watch=1 then call write_her("</ol>Description-cache index file ("outname") has been created.")
return 1


/**************************/
/* read swish file */
get_confile:

filelist.0=0

call syssleep 2         /* make sure things are properly closed */
foo=stream(daindx,'c','close')
foo=sref_open_read(daindx,30)
if foo<0 then do          /* couldn't open, give up */
    say "GOSWISH: could not open " daindx
    return 0
end /* do */

/* set some stuff fron daindx */
do mm=1 to 3
   foo=linein(daindx)
end /* do */
wow=linein(daindx)
parse var wow . ',' nfiles .; nfiles=strip(nfiles)


/* count lines in the file */
call linein daindx,1,0
ndo=0
do until lines(daindx)=0
   foo=linein(daindx)
   ndo=ndo+1
end /* do */
/* now get the lines ndo-nfiles to ndo-1 */
call linein daindx,1,0
do ij=1 to ndo-(1+nfiles)
   foo=linein(daindx)
end /* do */
/* convert url style names back to original files */
do nf=1 to nfiles
   baa=linein(daindx)
   parse var baa aa  '"' atitle '"' asize .
   afil=translate(strip(word(aa,1)))
   filelist.nf.!original=afil
   filelist.nf.!title=atitle
   filelist.nf.!size=asize
   do il=1 to nreps             /* convert to fully qualified names */
       if abbrev(afil,reprules.il.!new)=1 then do
             aa=reprules.il.!orig
             bb=substr(afil,1+length(reprules.il.!new))
             aa=aa||bb
             leave
       end  /* Do */
   end /* do */
   filelist.nf=aa

end /* do */
filelist.0=nfiles
return nfiles


/****************************/
/* given a filefilst, get descriptions */
get_filelist_info:
/* determine type of file: 2=text, 1=html, 0=non-text */
htmls=translate(translate(htmls),' ','.')
nocontents=translate(translate(nocontents),' ','.')
do mm=1 to filelist.0
   aff=filelist.mm
   filelist.mm.!type=2               /* assume it's text */
   foo=lastpos('.',aff)
   if foo=0 then iterate

   anext=strip(translate(substr(aff,foo+1)))
   if wordpos(anext,htmls)>0 then do
        filelist.mm.!type=1
        iterate
   end
   if wordpos(anext,nocontents)>0 then filelist.mm.!type=0
 
end /* do */

return 0



/***************/
/* ------------------------------------- */
/* get summary info, or get explicit description in fdescribe (DESCRIBE.TXT)
file 
afilename: fully qualified filename to investigate
atype: 1- html, 2-non-html text, 0-non text 
asummary: 1- explicit, 2-create

returns a text or html summary, or a numeric code:
1= File not available
2= Summary not available
3= Explicit summary not available
4= Error in routine -- no summary available

yaman is also returned:
 0-no description, 1=created, 2=explicit 
 
*/

make_summary:procedure  expose yaman atitle asize fdescribe latestd. continuation_flag

parse arg afilename,atype,asummary

gmess.1=' File not available'
gmess.2=' Summary not available'
gmess.3=' Explicit summary not available'
gmess.4=' No summary available'


yaman=0
eek=stream(afilename,'c','query exists')   /* check for existence*/

if eek="" then return gmess.4            /* error */

/* check for explicit description file */
if fdescribe<>" " then do
 checkd=strip(translate(strip(filespec('d',afilename)||filespec('p',afilename),'t','\')'\'))
 if checkd<> latestd.!dir then do 
    call make_desc(checkd)      /* saves to latestd. */
    latestd.!dir=checkd
 end

 fnm=strip(translate(filespec('n',afilename)))   /* check the descriptions, and return match if found */
 igoo=0
 do mm=1 to latestd.0
    if abbrev(latestd.mm,fnm)=1 then do 
        igoo=mm ; leave
    end /* do */
 end
 if igoo>0 then do
       iif=igoo
       yaman=2
       return latestd.iif.!desc
 end
end             /* check description file */


/* no explicit description -- perhaps create one? */
select

  when atype=0 then            /* not text, so no description */
      return gmess.2
 
  when atype=2 & asummary<>2 then   /* non-html text, non-create mode */
      return gmess.2


  when atype=2 & asummary=2 then do   /* non-html text, create mode */
       alen=min(chars(afilename),300)
       stuff=charin(afilename,1,alen)
       fpp=stream(afilename,'c','close')
       yaman=1
       wow=replacestrg(wow,'&','&amp;','ALL')
       wow=replacestrg(stuff,'<','&lt;','ALL')
       wow=replacestrg(wow,'>','&gt;','ALL')
       wow=replacestrg(wow,'"','&quot;','ALL')
       return wow
  end

  when atype=1 then do
     alen=min(chars(afilename),10000)
     stuff=charin(afilename,1,alen)
     fpp=stream(afilename,'c','close')
     stuff=space(translate(stuff,' ','00090a0d1a1b'x))
     wow=look_header(afilename)
     if wow<>0 then do
         yaman=2
        return wow
     end  /* Do */

    if wow=0 & asummary<>2 then 
       return gmess.4

    WOW=LOOK_HTAG()                     /* use <Hn> for summary */
    if wow<>0 then do 
       yaman=1
       return wow
    end  /* Do */
    return gmess.3
  end
 
  otherwise do
     say " ERROR: should not be here in make summary "
     return gmess.4
  end
end


/******************/
/* read a description file with possible continuation lines */
make_desc:procedure expose continuation_flag latestd.  fdescribe
parse arg checkd
 foo2=checkd||fdescribe
 if stream(foo2,'c','query exists')="" then do /*no such file */
     latestd.0=0
     return 
 end /* do */
 yyo=charin(foo2,1,chars(foo2)) ;foo1=stream(foo2,'c','close')
 im=0
 do until yyo=''
    im=im+1
    parse var yyo tmps.im '0d0a'x yyo
 end /* do */
 tmps.0=im
 igoo=0
 daflag=left(strip(continuation_flag),1)

/* continuation lines have daflag (typically |) as first non-space character */
ison=0 ; igoo=0
do mm=1 to tmps.0
   al=strip(tmps.mm)
   if ison=1 &   abbrev(al,daflag)=1  & daflag<>" " then do    /* this is acontinuation */
            parse var al (daflag) eek
            intos.igoo=intos.igoo||' '||eek
   end  /* Do */
  if al=" " | abbrev(al,';')=1 then do 
      ison=0 ; iterate
  end  /* Do */
  igoo=igoo+1
  intos.igoo=al
  ison=1   
end
intos.0=igoo

igoo=0
do ji=1 to intos.0
   igoo=igoo+1
   parse var intos.ji latestd.igoo latestd.igoo.!desc
   latestd.igoo=strip(translate(latestd.igoo))
end             /* refresh description file */
latestd.0=igoo

return igoo



/* ----------------------------------------------------------------------- */
/* Look for "desc" field in header     */
/* ----------------------------------------------------------------------- */

look_header: procedure expose stuff url_title
parse arg afile
dowrite=0
do until stuff=""

    parse var stuff  p1 '<' tag '>' stuff
    if  translate(word(tag,1))="HEAD" then do   /* now in head !*/
            dowrite=1
            iterate
    end
    if dowrite=0 then iterate    /* wait till we get into head .. */

    if  translate(word(tag,1))="/HEAD" then  /* out of head, all done ! */
        leave

/* IT IS A TITLE TAG?  */
     if translate(word(tag,1))="TITLE" then do
        parse var stuff url_title '<' footag '>' stuff
     end

/* is it a  META HTTP-EQUIV or a META NAME ? */
    if translate(word(tag,1))="META" then do
        parse var tag ameta atype '=' rest
        tatype=translate(atype)
        if tatype="HTTP-EQUIV" | tatype="NAME" then do
           parse var rest aval1 rest
           REST=STRIP(REST)

           aval1=strip(aval1) ;
           aval1=strip(aval1,,'"')
           if abbrev(translate(aval1),'DESC')<>1 then iterate

           aval2=" "
           foo1=ABBREV(translate(rest),'CONTENT')
           if foo1>0 then do
                PARSE VAR REST FOO '=' AVAL2
                aval2=strip(aval2)
                aval2=strip(aval2,'b','"')
                WOW=LEFT(AVAL2,500)
                return WOW
           end
        end             /* name or http-equiv */
    end         /* meta */
end             /* stuff */


return 0


/* ----------------------------------------------------------------------- */
/* Extract <hn> fields     */
/* ----------------------------------------------------------------------- */

look_htag: procedure expose stuff filename

stuff0=left(stuff,1000)

amessage=""
dowrite=0
do until stuff=""
    parse var stuff  p1 '<' tag '>' stuff
    ttag=translate(word(tag,1))
    if wordpos(ttag,' H1 H2 H3 H4 TITLE')>0 THEN DO   /* grab stuff */
        parse var stuff  amess '<' tag2 '>' stuff
        amessage=amessage||amess||'<b> | </b>'
    end
end

if amessage="" then do  /* getting desperate -- grab any old words! */
   do until stuff0=""
      parse var stuff0 p1 '<' tag '>' stuff0
      amessage=amessage||' '||p1
   end
end

if amessage="" then
   return 0
amessage=left(amessage,300)  /* keep it short */
return amessage




/* ------------- */
/* ----------------------------------------------------------------------- */
/* REPLACESTRG: In string astring, find first occurence substring target and
.   replace it with substring putme
.      if no target, return unchanged astring
.      if no putme, then remove target
.      if type=backward, then find/change LAST occurence
.      if type=all, find/change all occurences
.      if exactmatch=yes, then do not capitalize during search (exact match only */
/* ----------------------------------------------------------------------- */

replacestrg:procedure

exactmatch=0
backward=0 ; doall=0

parse arg astring ,  target   , putme , type , exactmatch

type = translate(type)
if type="BACKWARD" then backward="YES"
if type="ALL" then doall="YES"

iat=1
joelen=length(target)
joelen2=length(putme)

doagain:                /* here if doall=yes */
 if exactmatch="YES" then do
    if   backward="YES" then
        joe= lastpos(target,astring)
    else
        joe= pos(target,astring,iat)
 end
 else do
   if   backward="YES" then
        joe= lastpos(translate(target),translate(astring))
    else
        joe= pos(translate(target),translate(astring),iat)
 end
 if joe=0 then
         return astring

 astring=delstr(astring,joe,joelen)
 if putme<>' ' then
    astring=insert(putme,astring,joe-1)

 if doall="YES" then do
     iat=joe+joelen2
     signal doagain
 end
/* else, all done */
 return astring



/*********************************************/
/* SHOW list of SEARCH FORMS   */
/*********************************************/
show_Forms:procedure expose index_dir vs. is_cgi
parse arg list

crlf='0d0a'x

if is_cgi=1 then do
   call dumpit('content-type: text/html')
   call dumpit("")
end /* do */


dafile='<html><head><title>List of SWISH search forms</title></head></body>'crlf
dafile=dafile||'<a name="TOP"><h2>List of SWISH search forms </h2></a>'crlf

goindx=stream(index_dir'\goswish.ind','c','query exists')
if goindx<>"" then 
   yow=charin(goindx,1,chars(goindx))
else
  yow=""

nin=0
do until yow=""
   parse var yow aline (crlf) yow ; aline=strip(aline)
   if aline="" then iterate
   if abbrev(aline,';')=1 then iterate
   parse var aline theurl','thefile','theswi','thetime','thedesc
   if thefile='' | theswi='' | theurl='' then iterate
   if stream(thefile,'c','query exists')='' then iterate  /* relevant files still exist? */
   if stream(theswi,'c','query exists')='' then iterate

   nin=nin+1
   outs.nin='<a href="'theurl'">'thedesc'</a> <em>( 'thetime ')</em>'
end
if nin=0 then do                /* no legit entries */
  dafile=dafile||' <b>Sorry.</b> Currently, no search forms are available.</BODY> </HTML> '
  if is_cgi=0 then do
      call write_her(dafile)
  end
  else do
     say dafile ; dafile=''
  end
  return 0
end

/* got some still alive forms --  present them in a list */
dafile=dafile||'<b> The following search indices are current available:</b><br><ul>'
if is_cgi=1 then do
   call dumpit(dafile); dafile=''
end /* do */
do mm=1 to nin
    if is_cgi=0 then do
       dafile=dafile||crlf||'<li>'outs.mm
    end
    else do
       call dumpit(crlf||'<li>'outs.mm) ; dafile=''
   end
end

if is_cgi=0 then do
       dafile=dafile||crlf||'</ul></body></html>'
end
else do
       call dumpit(crlf||'</ul></body></html>') ; dafile=''
end

if is_cgi=0 then do
   gaz=value('SREF_PREFIX',,'os2environment')
   if gaz='' then
     'VAR type text/html name dafile '
   else
     foo=sref_gos('VAR type text/html name dafile ',dafile)
end
else do
  call dumpit(dafile)
end

return 1


/*********************************************/
/*********************************************/
/*********************************************/
/* THIS IS THE SEARCH COMPONENT  */
/*********************************************/

search_it:procedure expose index_dir
parse arg list

/************* END USER CHANGABLE PARAMETERS ************/

signal on error name errarf ; signal on syntax name errarf ;

/*---   Load REXX libraries ----- */
/* Load up advanced REXX functions */
foo=rxfuncquery('sysloadfuncs')
if foo=1 then do
  call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  call SysLoadFuncs
end
if \RxFuncQuery("SockLoadFuncs") then nop
else do
       call RxFuncAdd "SockLoadFuncs","rxSock","SockLoadFuncs"
       call SockLoadFuncs
end


method = value("REQUEST_METHOD",,'os2environment')
if method="" then do
     say "This is meant to be run as a cgi-bin script "
end  /* Do */

servername=value("SERVER_NAME",,'os2environment')
crlf='0d0a'x

/* Begin the result body.*/
turkey=index_dir'\ST$?????.OUT'

TEMPOUT = systempfileName(turkey)

say "Content-type: text/html"
say


if tempout = "0" | tempout="" then do
     tt='<!doctype html public "-//IETF//DTD HTML 2.0//EN">' crlf ,
       "<html><head><title>Index search results </title></head>" crlf ,
       "<body> <STRONG> ERROR: Could not access working directory </STRONG>" crlf ,
      " </BODY> </HTML> "
     say tt
     return 0
end


keywords='help'
index_file="INDEX.SWI"
text_descrip_file=' '
descrip_file_type='TEXT'

ndofiles=20
swopts=' '
header_file=' '
isfilesearch=0          /* 0=ignore, 1= urlname search, 2=description search */
aheader="Search the site-index "
ncmt=0 ; door=0
summary=1
exists=0
do until list="" 


  parse var list v1 '&' list

  parse var v1 avar '=' aval ; avar=translate(avar) ;
  aval=decodekeyval(translate(aval,' ','+'))
  aval=strip(aval,,'"')
  if abbrev(avar,"KEYWORD")=1 then  do
      keywords=decodekeyval(translate(aval,' ','+'))
  end
  if avar='H1' | avar='HEADER'   then do
      aheader=decodekeyval(translate(aval,' ','+'))
  end
  if abbrev(avar,'HEADER_FILE')=1 then do
      header_file=decodekeyval(translate(aval,' ','+'))
  end

  if abbrev(avar,'INDEX')=1 then do
     if pos(':',aval)>0 | abbrev(strip(aval),'\')=1 then do
          indxfile=strip(aval)
     end /* do */
     else do
       indxfile=index_dir||'\'||strip(avar,'l','\')
     end
  end
  if abbrev(avar,'TEXT_DESCRIP_FILE') then do
     if pos(':',aval)>0 | abbrev(strip(aval),'\')=1 then do
          descrip_file=strip(aval)
     end /* do */
     else do
         descrip_file=index_dir||'\'||strip(aval,'l','\')
     end
     descrip_file_type='TEXT'
  end


  if abbrev(avar,'COMMENT')=1 then do
      ncmt=ncmt+1
      comments.ncmt=translate(aval, ' ','+'||'00090d0a'x) 
  end
  if abbrev(avar,'COND')=1 then do
        select
         when abbrev(translate(aval),'Y')=1 then
               door=' OR '
         when abbrev(translate(aval),'O')=1 then
               door=' OR '
          when abbrev(translate(aval),'N')=1 then
                door=' NOT '
         otherwise
                door=0
        end
  end

  if abbrev(avar,'SUMMA')=1 then do
        tt=translate(aval)
        summary=1
        if tt="NO" then
           summary=1
        if tt="YES" | tt=1 then
           summary=2
        if tt="CREATE"| tt=2 then
           summary=3
  end

  if abbrev(avar,'EXISTS')=1 then do
        tt=translate(aval)
        if wordpos(translate(tt),'YES Y 1')>0 then
           exists=1
        else
           exists=0
  end

  if abbrev(avar,'OPTION')=1 then do
      aval=strip(translate(aval,' ','+'))
      select
         when abbrev(translate(aval),'FILE')=1 then
            isfilesearch=1      /* filename */
         when abbrev(translate(aval),'SUMMARY')=1 then
            isfilesearch=2      /* summary */
         otherwise
            swopts=swopts||" "||aval
      end
      if abbrev(translate(aval),'-m')=1 then
           parse arg aval . ndofiles
  end
end             /* Options parsing */


/* load "Description cache"? */
dcache.0=0
if  descrip_file<>' ' then do
   foo=stream(descrip_file,'c','query exists')
   if foo<>' ' then do
          div=' &^%^& ' 
          div2=' #$*~~#$* '
          goofy=charin(descrip_file,1,chars(descrip_file))
          foo=stream(descrip_file,'c','close')
          ii=0 ;dcache.0=0
          do  until goofy=''
              parse var goofy aa (div2) goofy
              ii=ii+1 ; dcache.0=ii
              parse var aa dcache.ii.!gotit (div) dcache.ii (div) dcache.ii.!title (div) ,
                           dcache.ii.!size (div) dcache.ii.!yes
              dcache.ii.!create=dcache.ii.!yes
          end                   /* read lines from descirpfile */
   end                   /* descrip file exisst */

   do mm=1 to dcache.0
       dcachel.mm=dcache.mm
   end /* do */
   dcachel.0=translate(dcache.0)
end  /* Do */

if datatype(ndofiles)<>'NUM' then ndofiles=20

/* if filesearch and no descriple, error */
if isfilesearch>0 & dcache.0=0 then do
  tt='<!doctype html public "-//IETF//DTD HTML 2.0//EN">' crlf ,
     "<html><head><title>Index search results </title></head>" crlf ,
    " <STRONG>Sorry</strong> FileSearch requires a description file.  </BODY> </HTML> " crlf 
  say tt
  return 0
end  /* Do */


aa=""
aa=aa|| '<!doctype html public "-//IETF//DTD HTML 2.0//EN">'
aa=aa|| "<html><head><title>Index search results </title>"
aa=aa||"</head>"
call dumpit(aa)


/* write the header -- use HEADER and COMMENTS */
aa=""
if header_file<>' ' then do
   wow=afileread(header_file,ahead,,'E')
   do ww=1 to wow
      aa=aa||crlf||ffread.ww
   end /* do */
end  /* Do */
if aa="" then do 
   aa='<body> <h1> ' aheader ' </h1> 'crlf
end
call dumpit(aa)

/* write out comments */
aa=""
do mm=1 to ncmt
   aa=aa||' <em> ' comments.mm ' </em> <br>'crlf
end
call dumpit(aa)

/* fix up keyword list */
twords=translate(keywords) ; srchwords=""
/* remove silly srchwords */
do mm=1 to words(twords)
   aword=word(twords,mm)
   if wordpos(aword,'AND NOT OR')>0 then iterate
   srchwords=srchwords||" "||aword
end

if door<>0 & words(keywords)>1 then do   /* insert not / or  into keyword list */
   tmp=word(keywords,1) ; wasand=0
   do mmm=2 to words(keywords)
      aww=word(keywords,mmm) ; taw=strip(translate(aww))
      if wasand=1 then do
        tmp=tmp||' '||aww
        wasand=0
        iterate
      end
      if taw="OR"| taw="AND" | taw="NOT"then do
        tmp=tmp||' '||aww
        wasand=1
        iterate
      end
      tmp=tmp||door||aww
    end
    keywords=tmp
end

if isfilesearch=0 then do
   if stream(indxfile,'c','query exists')="" then do
      aa=" <p><b>Sorry</b>, the requested search index does not exist (" indxfile ")  </BODY> </HTML> "
      call dumpit(aa)
      rmessage=' Error: No .SWI file: 'indxfile
      signal sendher
   end
   t1='swish -f '||INDXfile ||' -w '||keywords||' '||swopts||' > '||tempout
   foodir=directory(index_dir)
   address cmd
    t1
   address
   foodir=directory(foodir)

/* get results */
   gotem=afileread(tempout,'filelines',,'e')
   do iij=1 to gotem
      filelines.iij=ffread.iij
   end /* do */
   filelines.0=gotem
   goo=sysfiledelete(tempout)
end
else do         /* search filenames in */
  gotem=do_fsearch(isfilesearch,keywords,ndofiles)  /* one of the cache search variants */
end


if gotem=0 then do
    aa=" <p><STRONG> No matches found </STRONG> </BODY> </HTML> "
    say aa
    return  0
end
igot=0


do mm=1 to gotem
   RLINE = filelines.mm
   IF RLINE = '' THEN leave
   IF RLINE = '.' THEN leave


/* Skip comment lines and empty lines. */
   IF abbrev(RLINE,'#') = 1  THEN iterate

/* Split the line into fields. */
    PARSE VAR RLINE R_SCORE R_FILE R_stuff
    foo1=words(r_stuff)
    r_title=delword(r_stuff,foo1)
    r_position=word(r_stuff,foo1)

/* Make sure this looks valid (i.e. that position and score are both 
   numeric variables).                                               */

  IF VERIFY(R_SCORE, '1234567890') <> 0 | VERIFY(R_POSITION, '1234567890') <> 0 THEN  do
       aa=strip(rline)
       if abbrev(translate(aa),'SEARCH WORDS:') then do
            bb='<p><strong> Searching for: </strong> 'crlf
            parse var rline foo ':' lookme
            bb=bb||' <code> ' lookme '</code>'
            call dumpit(bb)
       end
       iterate
  end
  igot=igot+1
  if igot=1 then do
        aa='<h3> Results of search </h3>  <dl>'
        call dumpit(aa)
   end

/* Spit out this reference as a HTML link. */
  r_file=translate(r_file,'/','\')
  r_title=strip(strip(r_title),,'"')
  if r_title="" then do 
      tmp2=translate(r_file,' ','\/'); r_title=word(tmp2,words(tmp2))
  end /* do */
  aa2='<dt> <A href=' r_file '>' r_title '</a>'crlf

  aa='&#32; <em> Score= ' r_score ' </em> 'crlf
  aa=aa||' <code> , ' r_position 'bytes </code> <br>'crlf
  if summary>1 | exists>0 then do
     foo=make_summary2(aa,r_title,r_file,summary,srchwords,exists)
  end
  else do
     call dumpit(aa2||aa)
  end /* do */

end
if igot>0 then call dumpit('</dl>')

if gotit>0 then call dumpit("<p> <em> Total of " igot " matches </em>")

call dumpit('<p>')

sendher: nop
aa='</body></html> '
say aa
return 0


/*******************/
/* say to stdout */
dumpit:procedure
parse arg aa
crlf='0d0a'x
aa=aa||crlf
say aa
return 0


/************/
/* read file into ffread stem var */
afileread:procedure expose ffread.
parse arg hfile
crlf='0d0a'x
if stream(hfile,'c','query exists')="" then return 0
tmp=strip(charin(hfile,1,chars(hfile)),'t','1a'x)
tt=stream(hfile,'c','close')
itmp=0
do until tmp=""
   itmp=itmp+1
   parse var tmp ffread.itmp (crlf) tmp
end /* do */
ffread.0=itmp
return itmp 



/***********************************/
/* search files names, etc instead of keyword index 
dcache.  contains cache index 
dcachel. just the urls (for searching in)
filelines. return these to be displayed
isfilesearch: 0= normal keywords, 1=search files, 2=search summarys
keywords: keywords to search for (and highlight)
ndofiles: max matches to display
*/
do_fsearch:procedure expose dcache. dcachel. filelines.

  parse arg  isfilesearch,keywords,ndofiles
 keywords='OR '||translate(strip(keywords))
/* seperate out OR AND and NOT keywords. Default is AND (first
one is OR) */
alist.1=' ' ; alist.2=' ' ; alist.3=' ';ismod=1
do mm=1 to words(keywords)
   aw=strip(word(keywords,mm))
   oop= wordpos(aw,'OR AND NOT')
   if oop=0 then do
        alist.ismod=alist.ismod||' '||aw
        ismod=2                 /* reset to AND */
   end
   else do
        ismod=oop
   end  /* Do */
end /* do */

oof=keywords
if strip(translate(word(keywords,1)))="OR" then oof=subword(keywords,2)

if isfilesearch=1 then
   bb='<p><strong> Searching file names for: </strong><code> 'oof||'</code>'||'0d0a'x
else
   bb='<p><strong> Searching <em>summaries </em> for: </strong><code> 'oof||'</code>'||'0d0a'x

 call dumpit(bb)

ors=alist.1 ; ors.0=0
if ors<>' ' then ors.0=words(ors)
ands=alist.2 ; ands.0=0
if ands<>' ' then ands.0=words(ands)
nots=alist.3 ; nots.0=0
if nots<>' ' then nots.0=words(nots)

/* check each line for ors, ands, and nots */
do mm=1 to dcache.0
   isok.mm=0            /* assume faiulre */
 
   if isfilesearch=1 then
        aurl=strip(translate(dcachel.mm))
   else
        aurl=strip(translate(dcache.mm.!create))

/* if it's a not, failure */
   do mm2=1 to nots.0
      if pos(strip(word(nots,mm2)),aurl)>0 then iterate mm
   end
/* all and's have to be there */
    do mm2=1 to ands.0
        if pos(strip(word(ands,mm2)),aurl)=0 then iterate mm
    end /* do */
/*any of the ors */
   isok.mm=1           
   if ors.0=0  then iterate   /* >0 ors, 1 must be fulfilled */
   do mm2=1 to ors.0
      jj=strip(word(ors,mm2)) 
      if  pos(jj,aurl)>0 then iterate mm /* success*/
   end /* do */
   isok.mm=0             /* did not match an ors */
end /* do */

/* if isok=1, then this is a match. Return the first ndo matches in
the filelines. stem variable. Filelines. should contain:
 score url title position:
score=100 (that is, no score)
url=dcache.n
title=dcache.n.!title
bytes=title.n.!size
*/
noks=0
do mm=1 to dcache.0
   if isok.mm=1 then do
       noks=noks+1
       atitle=dcache.mm.!title ; if atitle="" then atitle=dcache.mm
       filelines.noks='100 ' dcache.mm ' ' atitle ' ' dcache.mm.!size
   end  /* Do */
   if noks>ndofiles then leave
end /* do */
filelines.0=noks

return noks
 

/* ---------------------------------------------------------------------- */
/* write summary info
Asummary:  2=explicit, 3=create
acheck:1=check on existence of file or url
*/

make_summary2:procedure expose tempfile ddir enmadd transaction  dcache. dcachel. 

parse arg aa,a_title,aurl,asummary, srchwords,acheck
crlf='0a0d'x

aayes='<dt> <A href=' aurl '>' a_title '</a>'crlf
aano='<dt> <u>' a_title '</u>'crlf

ishttp=abbrev(strip(translate(aurl)),"HTTP://") /* file, or url? */

/* if acheck=1, then check on existence of file, or of url */
if acheck=1 then do
  if ishttp=0 then do
    afilenam=aurl
    if afilenam<>"" then afilenam=stream(afilenam,'c','query exists')
    if afilename="" then
       call dumpit(aano||aa)
    else
       call dumpit(aayes||aa)
    
  end
  else do               /* use HEAD request for url */
     stuff=a_head_url(aurl)
     if stuff=0 then
       call dumpit(aano||aa)
     else
       call dumpit(aayes||aa)
  end /* do */
end
else do         /* no check, assume it exists */
       call dumpit(aayes||aa)
end

if asummary<2  then return 0            /* no summary, so return */

/* try and find/create a summary */

/* note: .!gotit values:  0-none, 1=created, 2=explicit */
aurl=translate(aurl)
if dcache.0>0  & asummary>1 then do           /* cache exists: Use it! */
   gotit=0
   do mm=1 to dcachel.0
       if dcachel.mm=aurl then do
                gotit=mm ; leave
       end /* do */
   end /* do */
   if gotit=0 then do
       call dumpit(' <dd> <code> Summary not available </code> ')
       return 0
    end
   ii=gotit  
   if asummary=2 then 
        dathing=strip(dcache.ii.!yes)
   else
        dathing=strip(dcache.ii.!create)
   if datatype(dathing)="NUM" then do   /* replace special codes */
        if dathing>0 & dathing<5 then dathing=gmess.dathing
   end  /* Do */
   if dcache.ii.!gotit=0 then do        /* no match signal */
      call dumpit('<dd> <code> '|| dathing|| ' </code> ')
      return 0
   end  /* Do */
   wow=space(translate(dathing,' ','00090a0d1a1b'x))    /* cleanup selection */
    do jmm=1 to words(srchwords)
         aword=strip(word(srchwords,jmm))
         wow=make_block(aword,wow,'<b>','</b>')   /* highlight matches */
     end
 
     if  dcache.ii.!gotit=1 then 
          aa='<dd>  <code> ' wow ' </code>'
     else
          aa='<dd>   ' wow
     call dumpit(aa)
     return 0
end  /* CACHED description file */

atype=translate(mediatype(aurl))  /* get mime type */

/* strip out http://a.b.c/ */

if ishttp=0 then do

  if afilenam="" then do
     call dumpit('<dd> <code> No summary available </code> ')
     return 0
  end

  eek=sysfiletree(afilenam,'aflist','F')   /* check for existence*/
  if eek<>0 | aflist.0=0 then do            /* error */
      call dumpit('<dd> <code> Summary not available </code> ')
      return 0
  end

end

/* text/plain summary ... */
if atype="TEXT/PLAIN" then do

   if asummary=2 then do  /* text/plain can not have explicit */
     call dumpit('<dd> <code> Summary not available </code> ')
     return 0
   end
  
 /* asummary must be 3  -- create */
    if ishttp=1 then do
        stuff=get_url(aurl,500)
        if length(stuff)>500 then stuff=left(stuff,500)
        else
        if stuff=0 then do
           call dumpit('<dd> <code> Summary not available </code> ')
           return 0
        end
    end
    else do
       filename=aflist.1
       filename=strip(word(aflist.1,words(aflist.1)))
       alen=min(chars(filename),500)
       stuff=charin(filename,1,alen)
    end

    wow=space(translate(stuff,' ','00090a0d1a1b'x))
    wow=replacestrg(wow,'<','&lt;','ALL')
    wow=replacestrg(wow,'>','&gt;','ALL')
    wow=replacestrg(wow,'"','&quot;','ALL')
    wow=replacestrg(wow,'&','&amp;','ALL')
    do jmm=1 to words(srchwords)
         aword=strip(word(srchwords,jmm))
         wow=make_block(aword,wow,'<b>','</b>')   /* highlight matches */
     end
    call dumpit('<dd>   '|| wow)
    return 0
end

/* if not html (and not text/plain), return no summary*/
if atype<>"TEXT/HTML"  then do
     call dumpit('<dd> <code> Summary not available </code> ')
   return 0
end


/* if here--  html*/
/* and the url points to a legit file; read it in (up to 10000 characters */

if ishttp=1 then do
        stuff=get_url(aurl,10000)
        if stuff=0 then do
           call dumpit('<dd> <code> Summary not available </code> ')
           return 0
        end
end
else do
   filename=aflist.1
   filename=strip(word(aflist.1,words(aflist.1)))
   alen=min(chars(filename),10000)
   stuff=charin(filename,1,alen)
end

stuff=space(translate(stuff,' ','00090a0d1a1b'x))
url_title=0

wow=look_header(filename)

if wow<>0 then do
  do jmm=1 to words(srchwords)
      aword=strip(word(srchwords,jmm))
      wow=make_block(aword,wow,'<b>','</b>')   /* highlight matches */
  end
  call dumpit('<dd> '|| wow)
  return 0
end
if asummary<>3 then do           /* return if not CREATE */
   call dumpit('<dd> <code> Explicit summary not available</code> ')
   return 0
end

WOW=LOOK_HTAG()
if wow<>0 then do

  do jmm=1 to words(srchwords)
      aword=strip(word(srchwords,jmm))
      wow=make_block(aword,wow,'<b>','</b>')   /* highlight matches */
  end
  call dumpit('<dd> <code>'||  wow ||' </code>')
  return 0
end

if url_title<>0 then
   aa='<dd> <code> ' url_title ' </code> '
else
   aa='<dd> <code> Summary not available </code> '
call dumpit(aa)
return 0



/* do a head request */
a_head_url:procedure
parse arg aurl
crlf='0d0a'x

got=""
aurl=fix_url(aurl)
if abbrev(translate(aurl),'HTTP://')=1 then do
   aurl=substr(aurl,8)
end
parse var aurl server '/' request

family  ='AF_INET'
httpport=80

rc=sockgethostbyname(server, "serv.0")  /* get dotaddress of server */
if rc=0 then do;  return 0; end

dotserver=serv.0addr                    /* .. */

gosaddr.0family=family                  /* set up address */
gosaddr.0port  =httpport
gosaddr.0addr  =dotserver

gosock = SockSocket(family, "SOCK_STREAM", "IPPROTO_TCP")

request=strip(request,'l','/')
message='HEAD /'request' HTTP/1.0'crlf'HOST:'server||crlf

message=message||crlf
got=''
rc = SockConnect(gosock,"gosaddr.0")
if rc<0 then do;  return 0 ; end
rc = SockSend(gosock, message)
/* Now wait for the response */
do r=1 by 1
  rc = SockRecv(gosock, "response", 1000)
  got=got||response
  if rc<=0 then leave
  end r
rc = SockClose(gosock)

parse var  got aa (crlf) .
parse var aa a1 a2 a3
if a2>=300 then return 0
return 1


/* ----------------------------------------------------------------------- */
/* MAKE BLOCK: Replace all occurences of NEEDLE in HAYSTACK
.        with delim1 needle delim2.
.        If delim1 and delim2 not give, then { AND } are used.
.   Example: make_block(boys,' there are wild boys out there','<b>',' </b>')
.      returns 'there are wild <b>boys </b> out  there'
.      (note that spaces are all retained)
*/
/* ----------------------------------------------------------------------- */

make_block:procedure

parse arg needle, haystack, delim1 , delim2, check_case
if delim1="" then delim1='{'
if delim2="" then delim1='}'

build=""
do forever
  if check_case<>1 then
     mm=pos(translate(needle),translate(haystack))
  else
     mm=pos(needle,haystack)

  if mm=0 then do
    build=build||haystack
    return build
  end

  t1=substr(haystack,1,mm-1)
  t2=substr(haystack,mm,length(needle))
  haystack=substr(haystack,mm+length(needle))
  build=build||t1||delim1||t2||delim2

end



/* ----------------------------------------------------------------------- */
/* MEDIATYPE: Return the media type of a file, based on its extension.     */
/* ----------------------------------------------------------------------- */
mediatype:procedure
parse arg aa,daport
  /* First get the extension; this assumes filenames have at least one '.' */
  ???=translate(substr(arg(1), lastpos('.',arg(1))+1))


  /* Fastpath very common types [add your own favourites] */
  if ???='HTM' then return 'text/html'
  if ???='HTML' then return 'text/html'
  if ???='GIF' then return 'image/gif'
  if ???='SHT' then return 'text/html'
  if ???='SHTML' then return 'text/html'



/* special cases */

  
  /* Set up the table of all types that we are interested in */
  known.   ='application/octet-stream'  /* default type */
  known.ps ='application/postscript'
  known.pdf='application/pdf'
  known.zip='application/zip'
  known.au ='audio/basic'
  known.snd='audio/basic'
  known.wav='audio/x-wav'
  known.mid='audio/x-midi'
  known.gif='image/gif'
  known.bmp='image/bmp'
  known.jpg='image/jpeg';  known.jpeg='image/jpeg'
  known.tif='image/tiff';  known.tiff='image/tiff'
  known.htm='text/html' ;  known.html='text/html'
  known.sht='text/html' ;  known.shtml='text/html'
  tmp1='HTML-SSI'           
  known.tmp1='text/html'   

  known.txt='text/plain'
  known.lst='text/plain'
  known.me='text/plain'
  known.log='text/plain'
  known.cmd='text/plain'
  known.doc='text/plain'
  known.in='text/plain'
  known.faq='text/plain'
  known.mpg='video/mpeg';  known.mpeg='video/mpeg'
  known.avi='video/x-msvideo'


  return known.???


/* ---------------------------------------------*/
/* get a url from some site, return first
maxchar characters (if maxchar missing, get 10million (the whole thing?) */
/* ---------------------------------------------*/
get_url:procedure
parse arg aurl,maxchar

if maxchar="" then maxchar=10000000

got=""
aurl=fix_url(aurl)
if abbrev(translate(aurl),'HTTP://')=1 then do
   aurl=substr(aurl,8)
end
parse var aurl server '/' request


/* now get the url.  This is based on GoServe's MOVEAUD command. It
requires the RxSock.DLL be in your LIBPATH. */

/* Load RxSock */
    if \RxFuncQuery("SockLoadFuncs") then nop
    else do
       call RxFuncAdd "SockLoadFuncs","rxSock","SockLoadFuncs"
       call SockLoadFuncs
    end

    crlf    ='0d0a'x                        /* constants */
    family  ='AF_INET'
    httpport=80

    rc=sockgethostbyname(server, "serv.0")  /* get dotaddress of server */
    if rc=0 then do
        return 0
    end
    dotserver=serv.0addr                    /* .. */
    gosaddr.0family=family                  /* set up address */
    gosaddr.0port  =httpport
    gosaddr.0addr  =dotserver

    gosock = SockSocket(family, "SOCK_STREAM", "IPPROTO_TCP")

    /* Set up request [HTTP 0.9 style, for all servers] */
    message="GET /"request''crlf

    got=''
    rc = SockConnect(gosock,"gosaddr.0")
    if rc<0 then do
        return 0
    end
    rc = SockSend(gosock, message)


 /* Now wait for the response */

   do r=1 by 1
     rc = SockRecv(gosock, "response", 1000)
     got=got||response
     if rc<=0 then leave
     tmplen=length(got)
     if tmplen> maxchar then leave
  end r

  rc = SockClose(gosock)

return got



/* ----------------------------------------------------------------------- */
/* FIX_URL: Make a fully specified http://url out of message            */
/* mayl not work if subdirectories have periods */
/* ----------------------------------------------------------------------- */
fix_url:procedure
parse arg message,servername,serverport

/* use defaults if not provided */
if servername="" then  servername=get_hostname()
if serverport="" then  serverport=80


 message=strip(translate(message,'/','\'))

 if abbrev(translate(message),"HTTP://")=1 then
             return message              /* assume the rest is legit */

/* if not a fully qualified http url (i.e. http://xxx.yyy/zzz) then
   make it so

Rule: (assuming  no http:// in message)
  Strip leading any leading /
  Look for a /  If no slash found, 
        look for periods.  If > 1 found, it's a "default" for a ip address
        if <2 found, it's a local file
  Check stuff before first /.
         If it has any periods, its an ip address (stuff after is the url)
         If no periods, it's a local url (stuff before is first subdirectory)
*/

message=strip(message,'l','/')


islash=pos('/',message)
if islash=0 then do
    foo=translate(message,' ','.')
    if words(foo)>2 then do
          anip=message
          aport=80
          afile=""
    end
    else do
          anip=servername
          aport=serverport
          afile=message
    end
end                                     /* no slashes found */

else do
   parse var message p1 '/' p2     /* slash found,extract what's before it */
   foo=translate(p1,' ','.')
   if words(foo)>1 then do    /* >0 periods signifies this is an ip address */
          anip=p1
          aport=80
          afile=p2
    end
    else do
          anip=servername
          aport=serverport
          afile=message
    end
end

isit="http://"||anip
if aport<>80 then
    isit=isit||':'||aport
isit=isit||'/'||afile

 return isit




/* get the hostname (aa.bb.cc) for this machine */                     
get_hostname: procedure                                                
    do queued(); pull .; end                   /* flush */             
    address cmd '@hostname | rxqueue'                                  
                                                                       
    parse pull hostname                                                
    return hostname                                                    
                                                                       

errarf:
say " ERROR at line " sigl
return 0
